@@ -4,13 +4,14 @@ from enum import Enum
 
 import pandas as pd
 import pendulum
-from pypika.terms import LiteralValue
-from pypika.functions import Lower
 from pypika import Criterion
+from pypika.functions import Lower
+from pypika.terms import LiteralValue
 
 from metrics_layer.core.exceptions import QueryError
 
 from .base import MetricsLayerBase
+from .week_start_day_types import WeekStartDayTypes
 
 
 class LiteralValueCriterion(Criterion):
@@ -104,13 +105,13 @@ class Filter(MetricsLayerBase):
     week_start_day_default = pendulum.MONDAY
     week_end_day_default = pendulum.SUNDAY
     week_start_day_lookup = {
-        "monday": pendulum.MONDAY,
-        "tuesday": pendulum.TUESDAY,
-        "wednesday": pendulum.WEDNESDAY,
-        "thursday": pendulum.THURSDAY,
-        "friday": pendulum.FRIDAY,
-        "saturday": pendulum.SATURDAY,
-        "sunday": pendulum.SUNDAY,
+        WeekStartDayTypes.monday: pendulum.MONDAY,
+        WeekStartDayTypes.tuesday: pendulum.TUESDAY,
+        WeekStartDayTypes.wednesday: pendulum.WEDNESDAY,
+        WeekStartDayTypes.thursday: pendulum.THURSDAY,
+        WeekStartDayTypes.friday: pendulum.FRIDAY,
+        WeekStartDayTypes.saturday: pendulum.SATURDAY,
+        WeekStartDayTypes.sunday: pendulum.SUNDAY,
     }
 
     def __init__(self, definition: dict = {}) -> None:
@@ -175,6 +176,7 @@ class Filter(MetricsLayerBase):
             return date
         return Filter._date_to_string(date)
 
+    @staticmethod
     def _add_to_end_date(date, lag: int, date_part: str):
         plural_date_part = FilterInterval.plural(date_part)
         singular_date_part = FilterInterval.singular(date_part)
@@ -334,8 +336,13 @@ class Filter(MetricsLayerBase):
         first_word = str(value).split(" ")[0]
         first_two_words = " ".join(str(value).split(" ")[:2])
 
+        # Handle field to field comparison
+        if isinstance(value, LiteralValue):
+            expression = MetricsLayerFilterExpressionType.EqualTo
+            cleaned_value = value
+
         # Handle null conditional
-        if value == "NULL":
+        elif value == "NULL" or value is None:
             expression = MetricsLayerFilterExpressionType.IsNull
             cleaned_value = None
 
